//------------------------------------------------------------------------------
// File: ResamplerPlugin.cpp
//------------------------------------------------------------------------------

/*
 based on Secret Rabbit Code audio resampler v0.1.2
 Copyright (c) Erik de Castro Lopo

 http://www.mega-nerd.com/SRC/index.html
*/

/*
* Modified for ReClock by OGO :)
*
*/

#include <assert.h>

#include "ResamplerPlugin.h"
#include "common.h"
#include "samplerate.h"

class PrivateData
{
public:
	SRC_STATE* m_rs;

	PrivateData(): m_rs(0) 
	{}
	
	~PrivateData() 
	{
		if (m_rs)
			src_delete(m_rs);
	}
};

#define GETPRIVATEDATA ((PrivateData*)m_privateData)

ResamplerPlugin::ResamplerPlugin()
{ 
	m_privateData = new PrivateData;

	SetParameters(SRC_SINC_FASTEST, 1);
	SetRate(44100,44100);
}

ResamplerPlugin::~ResamplerPlugin()
{
	assert(m_privateData);
	delete m_privateData;
}

void ResamplerPlugin::SetParameters(int quality, int channelCount)
{
	assert(m_privateData);
	delete m_privateData;
	m_privateData = new PrivateData;

	m_channelCount = channelCount;
	if (channelCount)
	{
		int error;

		switch(quality)
		{
			case 0:
				GETPRIVATEDATA->m_rs = src_new (SRC_LINEAR, m_channelCount, &error);
				break;

			case 1:
				GETPRIVATEDATA->m_rs = src_new (SRC_ZERO_ORDER_HOLD, m_channelCount, &error);
				break;

			case 2:
				GETPRIVATEDATA->m_rs = src_new (SRC_SINC_FASTEST, m_channelCount, &error);
				break;

			case 3:
				GETPRIVATEDATA->m_rs = src_new (SRC_SINC_MEDIUM_QUALITY, m_channelCount, &error);
				break;

			case 4:
				GETPRIVATEDATA->m_rs = src_new (SRC_SINC_BEST_QUALITY, m_channelCount, &error);
				break;

			default:
				assert(0);
		}

		assert (error == SRC_ERR_NO_ERROR);
	}
}

bool ResamplerPlugin::SetRate(int inRate, int outRate)
{
	m_inRate = inRate;
	m_outRate = outRate;

	if (inRate && outRate)
		return (src_is_valid_ratio((double)m_outRate/(double)m_inRate) == SRC_TRUE);
	else
		return false;
}

void ResamplerPlugin::ProcessFlow(float *inBuffer, int inSamples, float *outBuffer, int *outSamples, int maxOutSamples)
{
	*outSamples = 0;

	if (m_channelCount && m_inRate && m_outRate && GETPRIVATEDATA && GETPRIVATEDATA->m_rs)
	{
		SRC_DATA l_data;
		
		l_data.data_in = inBuffer;
		l_data.input_frames = inSamples / m_channelCount;
		l_data.input_frames_used = 0;

		l_data.data_out = outBuffer;
		l_data.output_frames = maxOutSamples / m_channelCount;
		l_data.output_frames_gen = 0;

		l_data.src_ratio = (double)m_outRate/(double)m_inRate;

		l_data.end_of_input = 0;

		int error = src_process (GETPRIVATEDATA->m_rs, &l_data);

		if (error == SRC_ERR_NO_ERROR)
		{
			assert(l_data.input_frames_used == l_data.input_frames);
			*outSamples = l_data.output_frames_gen * m_channelCount;
		}
	}
}

void ResamplerPlugin::Flush()
{
	if (GETPRIVATEDATA && GETPRIVATEDATA->m_rs)
		src_reset(GETPRIVATEDATA->m_rs);
}
